/*
 * desbench.c - benchmark test for DES
 */

#include <stdio.h>
#include <string.h>
#if !unix
#include <stdlib.h>
#endif /* !unix */


#if unix
#include <sys/types.h>
#include <sys/times.h>
#define TIMES 1
#endif /* unix */
#include <time.h>

#include "des_crypt.h"


typedef char bool;
#define	YES	1
#define	NO	0
typedef unsigned char uchar;
#if !unix
typedef unsigned long ulong;
#endif /* !unix */


static char *Progname = "desbench";




void usage()
{
	fprintf(stderr, "Usage: %s key times\n", Progname);
}



int main(argc, argv)
int argc; char *argv[];
{
	int dflag = DES_ENCRYPT;
	long i, rep;
#if TIMES
	clock_t start, stop;
	struct tms tm;
#define HZ 60
	double t;
#else
	time_t t, start, stop;
#endif /* TIMES */
	uchar key[8], block[8];

	while (++argv, --argc > 0 && **argv == '-' && argv[0][1]) {
		switch (argv[0][1]) {
		case 'd':
			dflag = DES_DECRYPT;
			break;
		default:
			fprintf(stderr, "Bad option: %s\n", *argv);
			usage();
			exit(1);
		}
	}
	if (argc != 2) {
		usage();
		exit(1);
	}
	memset(key, 0, 8);
	memset(block, 0, 8);
	strncpy(key, *argv++, 8);
	rep = strtol(*argv, NULL, 0);
	des_setparity(key);
	des_setkey(key);
#if TIMES
	times(&tm);
	start = tm.tms_utime;
#else
	for (t = time(NULL); (start = time(NULL)) == t; )
		;
#endif /* TIMES */
	for (i = 0; i < rep; i++)
		des_crypt(block, dflag);
#if TIMES
	times(&tm);
	stop = tm.tms_utime;
#else
	stop = time(NULL);
#endif /* TIMES */
	printf("Final output = %02X:%02X:%02X:%02X:%02X:%02X:%02X:%02X\n",
		block[0], block[1], block[2], block[3],
		block[4], block[5], block[6], block[7]);
#if TIMES
	t = (double)(stop - start) / HZ;
	printf("%ld en(de)cryptions in %g seconds\n", rep, t);
	printf("%g msec per block", (double)t * 1000 / rep);
	if (t == 0)
		printf("\n");
	else
		printf(" (%g kbits/sec)\n", (double)(rep * 64) / (t * 1000));
#else
	t = stop - start;
	printf("%ld en(de)cryptions in %ld seconds\n", rep, t);
	printf("%g - %g msec per block",
		(double)t * 1000 / rep, (double)(t + 1) * 1000 / rep);
	if (t == 0)
		printf("\n");
	else
		printf(" (%g - %g kbits/sec)\n",
			(double)(rep * 64) / ((t + 1) * 1000),
			(double)(rep * 64) / (t * 1000));
#endif /* TIMES */
	exit(0);
}

